
#include <MsTimer2.h>
#include <EEPROM.h>
#include <TM1637Display.h>

#define Relay   9
/*-----------------------------------------------------------------------------------------------------*/
#define CLK     5
#define DIO     4

TM1637Display TM1637(CLK, DIO);                                               // Set up the 4-Digit Display.

//  uint8_t data[]      = {0xFF, 0xFF, 0xFF, 0xFF};
uint8_t data[]        = {0, 0, 0, 0};
byte Digit[]          = {0, 0, 0, 0, 0};                                      // Перем. для вывода на индик.

int  Digit_0[]        = {0xBF,0x86,0xDB,0xCF,0xE6,0xED,0xAD,0x87,0xFF,0xEF};
int  Digit_1[]        = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};
int  Digit_2[]        = {0x00,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};
int  Digit_3[]        = {0x00,0x40,0xCF,0xCF,0xF3,0xB9,0xEE,0xEE,0xBF,0x00};

int  Presso[]         = {0, 25, 0, 0};
byte Select           = 1;
int  Sec[]            = {0, 0, 0, 0};
byte Write            = 0;
byte Comp_ON          = 0;

#define KEY_Plus        6
#define KEY_Select      7
#define KEY_Minus       8
/*-----------------------------------------------------------------------------------------------------*/
#define Pressure        5
#define Ak              3
const float R1        = 16000;                          // Резисторы делителя напряжения на
const float R2        = 5100;                           // Входное напряжение до 18 вольт

const float Typ_Vbg   = 1.13; // 1.0 -- 1.2             // Эту константу необходимо откалибровать индивидуально

float Vcc             = 0.0;
float Max_Volt        = 0.0;
int   AKKU_Car;
/*-----------------------------------------------------------------------------------------------------*/
#define Relay_ON  digitalWrite(Relay, HIGH); Comp_ON = 1;
#define Relay_OFF digitalWrite(Relay, LOW);  Comp_ON = 0;
#define Dig_ON    for (byte i = 1; i <= 4; i++) Digit[i] = 1;
#define Dig_OFF   for (byte i = 1; i <= 4; i++) Digit[i] = 0;
#define Led_ON    Dig_ON  Display(); for (byte i = 0; i <= 3; i++) TM1637.setSegments(data + i, 1, i);
#define Led_OFF   Dig_OFF Display(); for (byte i = 0; i <= 3; i++) TM1637.setSegments(data + i, 1, i);
/*=======================================================================================================
                                  Преобразования слова в BCD значениe
=======================================================================================================*/
int   High               = 0;                            // Переменная для хранения старшей тетрады.
int   Low                = 0;                            // Переменная для хранения младшей тетрады.
int   Num[]              = {0, 0, 0, 0, 0};
#define Zero for (byte i = 1; i <= 4; i++) Num[i] = 0; Low = Num[0]; High = 0; Num[4] = Low;
#define Zero_H Low = High; High = 0;
#define WORD_to_BCD  Zero   while (Num[4] >= 10) NUM(4); Zero_H while (Num[3] >= 10) NUM(3); \
                     Zero_H while (Num[2] >= 10) NUM(2); Zero_H while (Num[1] >= 10) NUM(1); Num[0] = 0;
void NUM(byte Numb) {High++; Low -= 10; Num[Numb] = Low; Num[Numb - 1] = High;}
/*=======================================================================================================
                                              Инициализация
=======================================================================================================*/
void setup() 
{
  Serial.begin(115200);

  TM1637.setBrightness(0x0F);                           // Устанавливаем яркость дисплея
/*------------------------------------------------------------------------------------------------------/
                                    Oпределение опорного напряжения
/*-----------------------------------------------------------------------------------------------------*/
  analogReference(DEFAULT);                             // DEFAULT INTERNAL использовать Vcc как AREF
  delay(100);
  Vcc = Read_Vcc();
  Max_Volt = Vcc / (R2 / (R1 + R2));
  Serial.print("Vcc = "); Serial.print(Vcc); Serial.print(" :  Max_Volt = "); Serial.print(Max_Volt);
  analogWrite(Ak, 0);
  Read_Volt();
  Serial.print(" : AKKU_Car = "); Serial.println(AKKU_Car);
/*-----------------------------------------------------------------------------------------------------*/
  pinMode(KEY_Plus,   INPUT_PULLUP);
  pinMode(KEY_Minus,  INPUT_PULLUP);
  pinMode(KEY_Select, INPUT_PULLUP);
  pinMode(Relay, OUTPUT);
  digitalWrite(Relay, LOW);
  
//  EEPROM.update(1, Presso[1]);
  Presso[1] = EEPROM.read(1);
  MsTimer2::set(50, Timing_ISR);                        // Задаем период прерывания по таймеру 50 мс
  MsTimer2::start();                                    // Разрешаем прерывание по таймеру
  
  Read_Volt(); Display(); delay(1000); Select = 0;
}
/*=======================================================================================================
                                              Главный цикл
=======================================================================================================*/
void loop() 
{
  Read_Volt();

  Presso[2] = analogRead(5);
  float Volt = (Presso[2] * 4.53) / 1024.0;
  Presso[0] = (((3.0 * ((float)Volt - 0.47)) * 1000000.0) / 10e5) * 100;

  if (Presso[3] <= Presso[0]) {Presso[3] = Presso[0]; Sec[3] = 0;}

  Display();
  if (Presso[0] - 45 >= Presso[1] * 10) Relay_OFF
}
/*=======================================================================================================
                                              
=======================================================================================================*/
void Convert(int Temp)
{
  Num[0] = Temp;
  WORD_to_BCD for (byte i = 1; i <= 4; i++) Digit[i] = Num[i];
  Display();
}
/*=======================================================================================================
                                              Динамическая индикация
=======================================================================================================*/
void Display()
{
  if (Write == 0)
  {
    if (AKKU_Car >= 0)
    {
      for (byte i = 0; i <= 9; i++)                                 // Загружаем значения цифр от 0 до 9
      {
        if (Digit[1] == i) data[0] = Digit_2[i];
        if (Digit[2] == i) data[1] = Digit_0[i];
        if (Digit[3] == i) data[2] = Digit_1[i];
        if (Digit[4] == i) data[3] = Digit_1[i];
      }
    }
  }
  else
  {
    for (byte i = 0; i <= 9; i++)                                 // Загружаем значения цифр от 0 до 9
    {
      if (Digit[1] == i) data[0] = Digit_3[i];
      if (Digit[2] == i) data[1] = Digit_3[i];
      if (Digit[3] == i) data[2] = Digit_3[i];
      if (Digit[4] == i) data[3] = Digit_3[i];
    }
    Write = 0;
  }
}
/*=====================================================================================================*/
